@@ -4,7 +4,12 @@
 
 namespace Icinga\Module\Cube;
 
+use Icinga\Module\Cube\IcingaDb\IcingaDbCube;
 use Icinga\Web\View;
+use ipl\Stdlib\Filter;
+use ipl\Web\Url;
+use Generator;
+use Icinga\Data\Tree\TreeNode;
 
 /**
  * CubeRenderer base class
@@ -79,6 +84,13 @@ abstract class CubeRenderer
      */
     abstract protected function getDetailsBaseUrl();
 
+    /**
+     * Get the severity sort columns
+     *
+     * @return Generator
+     */
+    abstract protected function getSeveritySortColumns(): Generator;
+
     /**
      * Initialize all we need
      */
@@ -185,13 +197,104 @@ abstract class CubeRenderer
         $this->view = $view;
         $this->initialize();
         $htm = $this->beginContainer();
-        foreach ($this->cube->fetchAll() as $row) {
+
+        $results = $this->cube->fetchAll();
+
+        if (! empty($results) && $this->cube::isUsingIcingaDb()) {
+            $sortBy = $this->cube->getSortBy();
+            if ($sortBy && $sortBy[0] === $this->cube::DIMENSION_SEVERITY_SORT_PARAM) {
+                $isSortDirDesc = isset($sortBy[1]) && $sortBy[1] !== 'asc';
+                $results = $this->sortBySeverity($results, $isSortDirDesc);
+            }
+        }
+
+        foreach ($results as $row) {
             $htm .= $this->renderRow($row);
         }
 
         return $htm . $this->closeDimensions() . $this->endContainer();
     }
 
+
+    /**
+     * Sort the results by severity
+     *
+     * @param $results array The fetched results
+     * @param $isSortDirDesc bool Whether the sort direction is descending
+     *
+     * @return Generator
+     */
+    private function sortBySeverity(array $results, bool $isSortDirDesc): Generator
+    {
+        $perspective = end($this->dimensionOrder);
+        $resultsCount = count($results);
+        $tree = [new TreeNode()];
+
+        $prepareHeaders = function (array $tree, object $row): TreeNode {
+            $node = (new TreeNode())
+                ->setValue($row);
+            $parent = end($tree);
+            $parent->appendChild($node);
+
+            return $node;
+        };
+
+        $i = 0;
+        do {
+            $row = $results[$i];
+            while ($row->$perspective === null) {
+                $tree[] = $prepareHeaders($tree, $row);
+
+                if (! isset($results[++$i])) {
+                    break;
+                }
+
+                $row = $results[$i];
+            }
+
+            for (; $i < $resultsCount; $i++) {
+                $row = $results[$i];
+
+                $anyNull = false;
+                foreach ($this->dimensionOrder as $dimension) {
+                    if ($row->$dimension === null) {
+                        $anyNull = true;
+                        array_pop($tree);
+                    }
+                }
+
+                if ($anyNull) {
+                    break;
+                }
+
+                $prepareHeaders($tree, $row);
+            }
+        } while ($i < $resultsCount);
+
+        $nodes = function (TreeNode $node) use (&$nodes, $isSortDirDesc): Generator {
+            yield $node->getValue();
+            $children = $node->getChildren();
+
+            uasort($children, function (TreeNode $a, TreeNode $b) use ($isSortDirDesc): int {
+                foreach ($this->getSeveritySortColumns() as $column) {
+                    $comparison = $a->getValue()->$column <=> $b->getValue()->$column;
+                    if ($comparison !== 0) {
+                        return $comparison * ($isSortDirDesc ? -1 : 1);
+                    }
+                }
+
+                // $a and $b are equal in terms of $priorities.
+                return 0;
+            });
+
+            foreach ($children as $node) {
+                yield from $nodes($node);
+            }
+        };
+
+        return $nodes($tree[1]);
+    }
+
     protected function renderRow($row)
     {
         $htm = '';
@@ -333,31 +436,34 @@ abstract class CubeRenderer
 
     protected function getDetailsUrl($name, $row)
     {
-        $cube = $this->cube;
+        $url = Url::fromPath($this->getDetailsBaseUrl());
 
-        $dimensions = array_merge(array_keys($cube->listDimensions()), $cube->listSlices());
-        $params = [
-            'dimensions' => DimensionParams::update($dimensions)->getParams()
-        ];
+        if ($this->cube instanceof IcingaDbCube && $this->cube->hasBaseFilter()) {
+            /** @var Filter\Rule $baseFilter */
+            $baseFilter = $this->cube->getBaseFilter();
+            $url->setFilter($baseFilter);
+        }
+
+        $urlParams = $url->getParams();
+
+        $dimensions = array_merge(array_keys($this->cube->listDimensions()), $this->cube->listSlices());
+        $urlParams->add('dimensions', DimensionParams::update($dimensions)->getParams());
 
         foreach ($this->cube->listDimensionsUpTo($name) as $dimensionName) {
-            $params[$dimensionName] = $row->$dimensionName;
+            $urlParams->add($this->cube::SLICE_PREFIX . $dimensionName, $row->$dimensionName);
         }
 
         foreach ($this->cube->getSlices() as $key => $val) {
-            $params[$key] = $val;
+            $urlParams->add($this->cube::SLICE_PREFIX . $key, $val);
         }
 
-        return $this->view->url(
-            $this->getDetailsBaseUrl(),
-            $params
-        );
+        return $url;
     }
 
     protected function getSliceUrl($name, $row)
     {
         return $this->view->url()
-            ->setParam($name, $row->$name);
+            ->setParam($this->cube::SLICE_PREFIX . $name, $row->$name);
     }
 
     protected function isOuterDimension($name)
