@@ -4,6 +4,7 @@
 
 namespace Icinga\Module\Cube\CubeRenderer;
 
+use Generator;
 use Icinga\Module\Cube\CubeRenderer;
 
 class ServiceStatusCubeRenderer extends CubeRenderer
@@ -17,26 +18,26 @@ class ServiceStatusCubeRenderer extends CubeRenderer
             $parts['critical'] = $facts->services_unhandled_critical;
         }
 
-        if ($facts->services_critical > 0 && $facts->services_critical > $facts->services_unhandled_critical) {
-            $parts['critical handled'] = $facts->services_critical - $facts->services_unhandled_critical;
+        if ($facts->services_unhandled_unknown > 0) {
+            $parts['unknown'] = $facts->services_unhandled_unknown;
         }
 
         if ($facts->services_unhandled_warning > 0) {
             $parts['warning'] = $facts->services_unhandled_warning;
         }
 
-        if ($facts->services_warning > 0 && $facts->services_warning > $facts->services_unhandled_warning) {
-            $parts['warning handled'] = $facts->services_warning - $facts->services_unhandled_warning;
-        }
-
-        if ($facts->services_unhandled_unknown > 0) {
-            $parts['unknown'] = $facts->services_unhandled_unknown;
+        if ($facts->services_critical > 0 && $facts->services_critical > $facts->services_unhandled_critical) {
+            $parts['critical handled'] = $facts->services_critical - $facts->services_unhandled_critical;
         }
 
         if ($facts->services_unknown > 0 && $facts->services_unknown > $facts->services_unhandled_unknown) {
             $parts['unknown handled'] = $facts->services_unknown - $facts->services_unhandled_unknown;
         }
 
+        if ($facts->services_warning > 0 && $facts->services_warning > $facts->services_unhandled_warning) {
+            $parts['warning handled'] = $facts->services_warning - $facts->services_unhandled_warning;
+        }
+
         if (
             $facts->services_cnt > $facts->services_critical && $facts->services_cnt > $facts->services_warning
             && $facts->services_cnt > $facts->services_unknown
@@ -48,6 +49,10 @@ class ServiceStatusCubeRenderer extends CubeRenderer
         $main = '';
         $sub = '';
         foreach ($parts as $class => $count) {
+            if ($count === 0) {
+                continue;
+            }
+
             if ($main === '') {
                 $main = $this->makeBadgeHtml($class, $count);
             } else {
@@ -83,28 +88,34 @@ class ServiceStatusCubeRenderer extends CubeRenderer
     protected function getDimensionClasses($name, $row)
     {
         $classes = parent::getDimensionClasses($name, $row);
-
         $sums = $row;
-        if ($sums->services_critical > 0) {
-            $classes[] = 'critical';
-            if ((int) $sums->services_unhandled_critical === 0) {
-                $classes[] = 'handled';
-            }
-        } elseif ($sums->services_warning > 0) {
-            $classes[] = 'warning';
-            if ((int) $sums->services_unhandled_warning === 0) {
-                $classes[] = 'handled';
-            }
-        } elseif ($sums->services_unknown > 0) {
-            $classes[] = 'unknown';
-            if ((int) $sums->services_unhandled_unknown === 0) {
-                $classes[] = 'handled';
-            }
-        } else {
-            $classes[] = 'ok';
+
+        $next = $this->cube->getDimensionAfter($name);
+        if ($next && isset($this->summaries->{$next->getName()})) {
+            $sums = $this->summaries->{$next->getName()};
         }
 
-        return $classes;
+        if ($sums->services_unhandled_critical > 0) {
+            $severityClass[] = 'critical';
+        } elseif ($sums->services_unhandled_unknown > 0) {
+            $severityClass[] = 'unknown';
+        } elseif ($sums->services_unhandled_warning > 0) {
+            $severityClass[] = 'warning';
+        }
+
+        if (empty($severityClass)) {
+            if ($sums->services_critical > 0) {
+                $severityClass = ['critical', 'handled'];
+            } elseif ($sums->services_unknown > 0) {
+                $severityClass = ['unknown', 'handled'];
+            } elseif ($sums->services_warning > 0) {
+                $severityClass = ['warning', 'handled'];
+            } else {
+                $severityClass[] = 'ok';
+            }
+        }
+
+        return array_merge($classes, $severityClass);
     }
 
     protected function makeBadgeHtml($class, $count)
@@ -123,4 +134,16 @@ class ServiceStatusCubeRenderer extends CubeRenderer
     {
         return 'cube/services/details';
     }
+
+    protected function getSeveritySortColumns(): Generator
+    {
+        $columns = ['critical', 'unknown', 'warning'];
+        foreach ($columns as $column) {
+            yield "services_unhandled_$column";
+        }
+
+        foreach ($columns as $column) {
+            yield "services_$column";
+        }
+    }
 }
