@@ -5,10 +5,10 @@
 namespace Icinga\Module\Cube\IcingaDb;
 
 use Icinga\Module\Cube\CubeRenderer\ServiceStatusCubeRenderer;
-use Icinga\Module\Icingadb\Model\CustomvarFlat;
 use Icinga\Module\Icingadb\Model\Service;
 use Icinga\Module\Icingadb\Model\ServicestateSummary;
 use ipl\Stdlib\Filter;
+use ipl\Stdlib\Str;
 
 class IcingaDbServiceStatusCube extends IcingaDbCube
 {
@@ -43,25 +43,10 @@ class IcingaDbServiceStatusCube extends IcingaDbCube
 
     public function listAvailableDimensions()
     {
-        $db = $this->getDb();
-
-        $query = CustomvarFlat::on($db);
-        $this->applyRestrictions($query);
-
-        $query
-            ->columns('flatname')
-            ->orderBy('flatname')
-            ->filter(Filter::like('service.id', '*'));
-        $query->getSelectBase()->groupBy('flatname');
-
-        $dimensions = [];
-        foreach ($query as $row) {
-            // Replaces array index notations with [*] to get results for arbitrary indexes
-            $name = preg_replace('/\\[\d+](?=\\.|$)/', '[*]', $row->flatname);
-            $dimensions[$name] = $name;
-        }
-
-        return $dimensions;
+        return array_merge(
+            $this->fetchServiceVariableDimensions(),
+            $this->fetchHostVariableDimensions()
+        );
     }
 
     public function prepareInnerQuery()
