@@ -84,25 +84,6 @@ class IdoServiceStatusCube extends IdoCube
         return $select;
     }
 
-    /**
-     * Add a specific named dimension
-     *
-     * Right now this are just custom vars, we might support group memberships
-     * or other properties in future
-     *
-     * @param string $name
-     *
-     * @return $this
-     */
-    public function addDimensionByName($name)
-    {
-        if (count($this->filterProtectedCustomvars([$name])) === 1) {
-            $this->addDimension($this->createDimension($name));
-        }
-
-        return $this;
-    }
-
     public function createDimension($name)
     {
         $this->registerAvailableDimensions();
